/*********************************************************************
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2008, Willow Garage, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Willow Garage nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *********************************************************************/

#ifndef RIQ_HAND_ETHERCAT_HARDWARE__ETHERCAT_AL_H
#define RIQ_HAND_ETHERCAT_HARDWARE__ETHERCAT_AL_H

#include "ethercat_hardware/ethercat_device.h"
#include "riq_msgs/RIQHandCommand.h"
#include <boost/thread/mutex.hpp>

namespace riq_hand_ethercat_hardware
{


struct ECatStates
{
  enum {INIT=0,BOOTSTRAP=3,PREOP=2,SAFEOP=4,OP=8};
  static const char* stateString(unsigned state);
};

struct ECatALStatus
{
  union {
    uint16_t raw_;
    struct {
      uint16_t state_ : 4;
      uint16_t error_ : 1;
      uint16_t res_   : 11;
    } __attribute__ ((__packed__));
  } __attribute__ ((__packed__));

  const char* stateString() const {return ECatStates::stateString(state_);}
  
  static const unsigned BASE_ADDR = 0x130;
} __attribute__ ((__packed__));


struct ECatALStatusCode
{
  uint16_t error_code_;

  static const char* errorCodeString(unsigned code);  
  const char* errorCodeString() const {return errorCodeString(error_code_);}

  enum {NO_ERROR=0, SYNC_MANAGER_WATCHDOG=0x1b};

  static const unsigned BASE_ADDR = 0x134;
} __attribute__ ((__packed__));


struct ECatALStatusAll
{
  static const unsigned BASE_ADDR = 0x130;
  ECatALStatus      status_;
  uint16_t          unused_;
  ECatALStatusCode  status_code_;

  bool readData(EthercatCom *com, EtherCAT_SlaveHandler *sh);

  static void sizeAssert();
} __attribute__ ((__packed__));


struct ECatALControl
{
  union {
    uint16_t raw_;
    struct {
      uint16_t state_ : 4;
      uint16_t ack_error_ : 1;
      uint16_t res_ : 11;
    } __attribute__ ((__packed__));
  } __attribute__ ((__packed__));

  static const unsigned BASE_ADDR = 0x120;
    
  bool writeData(EthercatCom *com, EtherCAT_SlaveHandler *sh) const;

  static void sizeAssert();
} __attribute__ ((__packed__));


}; //namespace riq_hand_ethercat_hardware

#endif /* RIQ_HAND_ETHERCAT_HARDWARE__ETHERCAT_AL_H */
