#include "riq_hand_ethercat_hardware/ethercat_al.h"
#include "ethercat_hardware/ethercat_device.h"

namespace riq_hand_ethercat_hardware
{

const char* ECatStates::stateString(unsigned state)
{
  switch(state)
  {
  case INIT: return "Init";
  case BOOTSTRAP: return "Bootstap";
  case PREOP: return "PreOP";
  case SAFEOP: return "SafeOP";
  case OP: return "OP";
  default: return "Invalid";
  }
}


const char* ECatALStatusCode::errorCodeString(unsigned code)
{
  switch(code)
  {
  case NO_ERROR:
    return "No error";
  case SYNC_MANAGER_WATCHDOG: 
    return "Sync manager watchdog";
  default: return "Unknown";
  }
}

bool ECatALStatusAll::readData(EthercatCom *com, EtherCAT_SlaveHandler *sh)
{
  int result = EthercatDevice::readData(com, sh, BASE_ADDR, this, sizeof(*this), EthercatDevice::FIXED_ADDR);
  if (result != 0)
  {
    return false;
  }
  return true;
}

void ECatALStatusAll::sizeAssert()
{
  BOOST_STATIC_ASSERT(sizeof(ECatALStatusAll) == 6);
}


bool ECatALControl::writeData(EthercatCom *com, EtherCAT_SlaveHandler *sh) const
{
  int result = EthercatDevice::writeData(com, sh, BASE_ADDR, this, sizeof(*this), EthercatDevice::FIXED_ADDR);
  if (result != 0)
  {
    return false;
  }
  return true;
}

void ECatALControl::sizeAssert()
{
  BOOST_STATIC_ASSERT(sizeof(ECatALControl) == 2);
}

}; //namespace riq_hand_ethercat_hardware

